require 'json'
require 'net/http'

# Client of kong's admin API.
module Kodmin
  # Error.
  class Error < StandardError
    def initialize(code, error)
      super("#{code}: #{error}")
    end
  end

  # HTTP client of API.
  module Client
    class << self
      attr_accessor :kong_url
    end

    def get(path, query_params = {})
      uri = uri_of(path, query_params)
      resp = Net::HTTP.start(uri.host, uri.port, nil, nil, nil, nil, use_ssl: (uri.scheme == 'https')) do |http|
        req = Net::HTTP::Get.new(uri.request_uri)
        set_request_headers(req)
        http.request(req)
      end
      check_and_return(resp)
    end

    def post(path, data = {})
      uri = uri_of(path)
      resp = Net::HTTP.start(uri.host, uri.port, nil, nil, nil, nil, use_ssl: (uri.scheme == 'https')) do |http|
        req = Net::HTTP::Post.new(uri.request_uri)
        set_request_headers(req)
        req['Content-Type'] = 'application/json'
        req.body = data.to_json
        http.request(req)
      end
      check_and_return(resp)
    end

    def put(path, data = {})
      uri = uri_of(path)
      resp = Net::HTTP.start(uri.host, uri.port, nil, nil, nil, nil, use_ssl: (uri.scheme == 'https')) do |http|
        req = Net::HTTP::Put.new(uri.request_uri)
        set_request_headers(req)
        req['Content-Type'] = 'application/json'
        req.body = data.to_json
        http.request(req)
      end
      check_and_return(resp)
    end

    def delete(path, query_params = {})
      uri = uri_of(path, query_params)
      resp = Net::HTTP.start(uri.host, uri.port, nil, nil, nil, nil, use_ssl: (uri.scheme == 'https')) do |http|
        req = Net::HTTP::Delete.new(uri.request_uri)
        set_request_headers(req)
        http.request(req)
      end
      check_and_return(resp)
    end

    private

    def uri_of(path, query_params = {})
      url = Client.kong_url + path
      if query_params.empty?
        URI(url)
      else
        queries = URI.encode_www_form(query_params)
        url.include?('?') ? URI(url + '&' + queries) : URI(url + '?' + queries)
      end
    end

    def set_request_headers(request)
      request[:Accept] = 'application/json'
    end

    def check_and_return(response)
      if response.is_a?(Net::HTTPSuccess)
        response.body
      else
        raise Error.new(response.code, response.body)
      end
    end
  end

  class << self
    # Initialize the Kong admin client.
    # @param kong_url [String] URL of the Kong Admin API, e.x. 'http://127.0.0.1:8001'.
    def init(kong_url)
      Client.kong_url = kong_url.delete_suffix('/')
    end
  end
end
