require 'singleton'
require 'securerandom'
require_relative 'client'

module Kodmin
  # Consumer of Kong API.
  class Consumer
    attr_accessor :id, :username, :custom_id, :created_at, :tags

    # New a consumer.
    # @param attributes [Hash] attributes of a consumer.
    def initialize(attributes = {})
      from_hash(attributes)
    end

    # Assign attributes from a Hash.
    # @param attributes [Hash] attributes.
    def from_hash(attributes)
      attributes.each_key do |key|
        setter = "#{key}=".to_sym
        send(setter, attributes[key]) if respond_to?(setter)
      end
    end

    def to_hash
      hash = {}
      hash[:id] = id unless id.nil?
      hash[:username] = username unless username.nil?
      hash[:custom_id] = custom_id unless custom_id.nil?
      hash[:created_at] = created_at unless created_at.nil?
      hash[:tags] = tags unless tags.nil?
      hash
    end

    # Consumer's HMAC credential.
    class HmacAuth
      attr_accessor :id, :consumer, :username, :secret, :created_at

      # New a HmacAuth.
      # @param attributes [Hash] attributes of the HmacAuth.
      def initialize(attributes = {})
        from_hash(attributes)
      end

      # Assign attributes from a Hash object.
      # @param attributes [Hash] hash representation of attributes.
      def from_hash(attributes)
        attributes.each_key do |key|
          if key.to_sym == :consumer
            self.consumer = Consumer.new(attributes[key])
          else
            setter = "#{key}=".to_sym
            send(setter, attributes[key]) if respond_to?(setter)
          end
        end
      end

      def to_hash
        hash = {}
        hash[:id] = id unless id.nil?
        hash[:consumer] = consumer.to_hash unless consumer.nil?
        hash[:username] = username unless username.nil?
        hash[:secret] = secret unless secret.nil?
        hash[:created_at] = created_at unless created_at.nil?
        hash
      end
    end
  end

  # Operations on consumer.
  class Consumers
    include Singleton
    include Client

    # List a page of consumers.
    # @param offset [String] the offset token for pagination. An empty offset token refers to the first page.
    # @return [Hash] a Hash with keys of :consumers and :offset.
    def list_consumers(offset = '')
      params = offset.empty? ? {} : { offset: offset }
      resp = get('/consumers', params)
      resp = JSON.parse(resp)
      consumers = []
      resp['data'].each do |v|
        consumers << Consumer.new(v)
      end
      offset_token = nil
      unless resp['next'].nil? || resp['next'].empty?
        offset_token = resp['next'].split('=')[1]
      end
      { consumers: consumers, offset: offset_token }
    end

    # Create a new consumer.
    # @param attributes [Hash] attributes of the consumer.
    # @return [Consumer] the created consumer.
    def create_consumer(attributes)
      resp = post('/consumers', attributes)
      Consumer.new(JSON.parse(resp))
    end

    # Delete a consumer.
    # @param id [String] id of the consumer to be removed.
    def delete_consumer(id)
      delete("/consumers/#{id}")
    end

    # Get a consumers's HMAC credentials.
    # @param id [String] id of the consumer.
    # @param offset [String] the offset token for pagination. An empty offset token refers to the first page.
    # @return [Hash] a Hash with keys of :credentials and :offset.
    def list_consumer_hmac_credentials(id, offset = '')
      params = offset.empty? ? {} : { offset: offset }
      resp = get("/consumers/#{id}/hmac-auth", params)
      resp = JSON.parse(resp)
      credentials = []
      resp['data'].each do |v|
        credentials << Consumer::HmacAuth.new(v)
      end
      offset_token = nil
      unless resp['next'].nil? || resp['next'].empty?
        offset_token = resp['next'].split('=')[1]
      end
      { credentials: credentials, offset: offset_token }
    end

    def create_consumer_hmac_credential(id)
      access_key = SecureRandom.hex(16)
      secret_key = SecureRandom.hex(16)
      resp = post("/consumers/#{id}/hmac-auth", {username: access_key, secret: secret_key})
      Consumer::HmacAuth.new(JSON.parse(resp))
    end
  end
end
